@@ -1,13 +1,11 @@
 // -*- coding: utf-8; -*-
 // Package WCom.Navigation
 WCom.Navigation = (function() {
-   const navId        = 'navigation';
-   const dsName       = 'navigationConfig';
+   const navId  = 'navigation';
+   const dsName = 'navigationConfig';
    class Navigation {
       constructor(container, config) {
          this.container        = container;
-         this.menus            = config['menus'];
-         this.messages         = new Messages(config['messages']);
          this.moniker          = config['moniker'];
          this.properties       = config['properties'];
          this.baseURL          = this.properties['base-url'];
@@ -28,11 +26,11 @@ WCom.Navigation = (function() {
          this.version          = this.properties['version'];
          this.contentContainer = document.getElementById(this.containerName);
          this.contentPanel     = document.getElementById(this.contentName);
-         this.contextPanels    = {};
-         this.content;
-         this.headerMenu;
-         this.globalMenu;
-         this.titleEntry;
+         this.menu             = new Menus(this, config['menus']);
+         this.messages         = new Messages(config['messages']);
+         this.titleEntry       = 'Loading';
+         const head = (document.getElementsByTagName('head'))[0];
+         this.titleElement     = head.querySelector('title');
          container.append(this.renderTitle());
          window.addEventListener('popstate', this.popstateHandler());
          window.addEventListener('resize', this.resizeHandler());
@@ -43,7 +41,8 @@ WCom.Navigation = (function() {
             const href = link.href + '';
             if (href.length && url == href.substring(0, url.length)
                 && !link.getAttribute('clicklistener')) {
-               link.addEventListener('click', this.loadLocation(href, options));
+               const handler = this.menu.clickHandler(href, options);
+               link.addEventListener('click', handler);
                link.setAttribute('clicklistener', true);
             }
          }
@@ -51,58 +50,11 @@ WCom.Navigation = (function() {
             const action = form.action + '';
             if (action.length && url == action.substring(0, url.length)
                 && !form.getAttribute('submitlistener')) {
-               form.addEventListener(
-                  'submit', this.submitHandler(form, options)
-               );
+               const handler = this.menu.submitHandler(form, options);
+               form.addEventListener('submit', handler);
             }
          }
       }
-      addSelected(item) {
-         item.classList.add('selected');
-         return true;
-      }
-      confirmHandler(name) {
-         return function(event) {
-            if (this.confirm) {
-               if (confirm(this.confirm.replace(/\*/, name))) return true;
-            }
-            else if (confirm()) return true;
-            event.preventDefault();
-            return false;
-         }.bind(this);
-      }
-      iconImage(icon) {
-         if (icon && icon.match(/:/)) {
-            return this.h.img({ src: icon });
-         }
-         else if (icon) {
-            const icons = this.icons;
-            if (!icons) return this.h.span({ className: 'text' }, '≡');
-            return this.h.icon({ className: 'icon', icons, name: icon });
-         }
-         return icon;
-      }
-      isCurrentHref(href) {
-         return history.state && history.state.href == href ? true : false;
-      }
-      loadLocation(href, options) {
-         return function(event) {
-            event.preventDefault();
-            if (options.renderLocation) options.renderLocation(href);
-            else this.renderLocation(href);
-         }.bind(this);
-      }
-      async loadMenuData(url) {
-         const state = { href: url + '' };
-         history.pushState(state, 'Unused', url); // API Darwin award
-         url.searchParams.set('navigation', true);
-         const { object } = await this.bitch.sucks(url);
-         if (!object || !object['menus']) return;
-         this.containerLayout = object['container-layout'];
-         this.menus = object['menus'];
-         this.token = object['verify-token'];
-         this.titleEntry = object['title-entry'];
-      }
       popstateHandler() {
          return function(event) {
             const state = event.state;
@@ -110,7 +62,7 @@ WCom.Navigation = (function() {
          }.bind(this);
       }
       async process(action, form) {
-         const options = { headers: { prefer: 'render=partial' }, form: form };
+         const options = { headers: { Prefer: 'render=partial' }, form: form };
          const { location, reload, text }
                = await this.bitch.blows(action, options);
          if (location) {
@@ -143,35 +95,16 @@ WCom.Navigation = (function() {
          }
          console.log('Recovered state ' + count + ' ' + state.href);
       }
-      async render() {
+      render() {
          this.messages.render(window.location.href);
-         this.renderMenu();
+         this.menu.render();
          this.scan(this.contentPanel);
       }
-      renderControl() {
-         if (!this.menus['_control']) return;
-         let attr = { className: 'nav-panel control-panel' };
-         const list = this.renderList(this.menus['_control'], 'control');
-         this.contextPanels['control'] = this.h.div(attr, list);
-         attr = { className: 'nav-control' };
-         const link = this.h.a(this.renderControlIcon());
-         return this.h.div(attr, [link, this.contextPanels['control']]);
-      }
-      renderControlIcon() {
-         const icons = this.icons;
-         if (!icons)
-            return this.h.span({ className: 'nav-control-label text' }, '≡');
-         const name = this.controlIcon || 'settings';
-         const icon = this.h.icon({
-            className: 'settings-icon', height: 24, icons, name, width: 24
-         });
-         return this.h.span({ className: 'nav-control-label' }, icon);
-      }
       async renderHTML(html) {
          let className = this.containerName;
          if (this.containerLayout) className += ' ' + this.containerLayout;
          this.contentContainer.setAttribute('class', className);
-         const attr  = { id: this.contentName, className: this.contentName };
+         const attr = { id: this.contentName, className: this.contentName };
          const panel = this.h.div(attr);
          panel.innerHTML = html;
          await this.scan(panel);
@@ -180,84 +113,16 @@ WCom.Navigation = (function() {
             this.contentContainer, 'contentPanel', panel
          );
       }
-      renderItem(item, menuName) {
-         const [text, href, icon] = item;
-         const iconImage = this.iconImage(icon);
-         const title = iconImage && this.linkDisplay == 'icon' ? text : '';
-         if (typeof text != 'object') {
-            const label = this.renderLabel(icon, text);
-            if (href) {
-               const attr = { href: href, onclick: this.loadLocation(href, {})};
-               const link = this.h.a(attr, label);
-               link.setAttribute('clicklistener', true);
-               return this.h.li({ className: menuName, title: title }, link);
-            }
-            const labelAttr = { className: 'drop-menu' };
-            const menuItem = this.h.span(labelAttr, label);
-            return this.h.li({ className: menuName, title: title }, menuItem);
-         }
-         if (!text || text['method'] != 'post') return;
-         const form = this.h.form({
-            action: href, className: 'inline', method: 'post'
-         }, this.h.hidden({ name: '_verify', value: this.token }));
-         form.addEventListener('submit', this.submitHandler(form, {}));
-         form.append(this.h.button({
-            className: 'form-button', onclick: this.confirmHandler(name)
-         }, this.h.span(this.renderLabel(icon, text['name']))));
-         return this.h.li({ className: menuName, title: title }, form);
-      }
-      renderLabel(icon, text) {
-         const iconImage = this.iconImage(icon);
-         return {
-            both: [iconImage, text],
-            icon: iconImage ? iconImage : text,
-            text: text
-         }[this.linkDisplay];
-      }
-      renderList(list, menuName) {
-         const [title, itemList] = list;
-         if (!itemList.length) return this.h.span();
-         const items = [];
-         let context = false;
-         let isSelected = false;
-         for (const item of itemList) {
-            if (typeof item == 'string' && this.menus[item]) {
-               let className = 'nav-panel';
-               if (menuName == 'context' || menuName == 'control')
-                  className = 'slide-out';
-               this.contextPanels[item] = this.h.div(
-                  { className }, this.renderList(this.menus[item], 'context')
-               );
-               context = item;
-               continue;
-            }
-            const listItem = this.renderItem(item, menuName);
-            if (context) {
-               const panel = this.contextPanels[context];
-               if (panel.firstChild.classList.contains('selected'))
-                  isSelected = this.addSelected(listItem);
-               listItem.append(panel);
-               context = false;
-            }
-            if (this.isCurrentHref(item[1]))
-               isSelected = this.addSelected(listItem);
-            items.push(listItem);
-         }
-         const navList = this.h.ul({ className: 'nav-list' }, items);
-         if (menuName) navList.classList.add(menuName);
-         if (isSelected) navList.classList.add('selected');
-         return navList;
-      }
       async renderLocation(href) {
          const url = new URL(href);
          url.searchParams.delete('mid');
          const opt = { headers: { prefer: 'render=partial' }, response: 'text'};
          const { location, text } = await this.bitch.sucks(url, opt);
          if (text && text.length > 0) {
-            await this.loadMenuData(url);
+            await this.menu.loadMenuData(url);
             await this.renderHTML(text);
             this.setHeadTitle();
-            this.renderMenu();
+            this.menu.render();
          }
          else if (location) {
             this.messages.render(location);
@@ -267,21 +132,8 @@ WCom.Navigation = (function() {
             console.warn('Neither content nor redirect in response to get');
          }
       }
-      renderMenu() {
-         if (!this.menus) return;
-         const content = [this.renderControl()];
-         if (!this.menus['_global']) return;
-         const global = this.renderList(this.menus['_global'], 'global');
-         if (this.location == 'header') content.unshift(global);
-         const cMenu = this.h.nav({ className: 'nav-menu' }, content);
-         this.headerMenu = this.display(this.container, 'headerMenu', cMenu);
-         if (this.location == 'header') return;
-         const container = document.getElementById(this.location);
-         const gMenu = this.h.nav({ className: 'nav-menu' }, global);
-         this.globalMenu = this.display(container, 'globalMenu', gMenu);
-      }
       renderTitle() {
-         const title = this.logo.length ? [this.iconImage(this.logo)] : [];
+         const title = this.logo.length ? [this.menu.iconImage(this.logo)] : [];
          title.push(this.h.span({ className: 'title-text' }, this.title));
          return this.h.div({ className: 'nav-title' }, title);
       }
@@ -308,34 +160,201 @@ WCom.Navigation = (function() {
                frame.classList.add('link-display-' + original);
                this.linkDisplay = original;
             }
-            if (linkDisplay != this.linkDisplay) this.renderMenu();
+            if (linkDisplay != this.linkDisplay) this.menu.render();
          }.bind(this);
       }
       async scan(panel, options = {}) {
-         for (const scanCallback of WCom.Util.Event.callbacks())
-            scanCallback(panel, options);
+         for (const scanCallback of WCom.Util.Event.onloadCallbacks())
+            await scanCallback(panel, options);
          this.addEventListeners(panel, options);
       }
       setHeadTitle() {
-         const head  = (document.getElementsByTagName('head'))[0];
-         const title = head.querySelector('title');
          const entry = this.capitalise(this.titleEntry);
-         title.innerHTML = this.titleAbbrev + ' - ' + entry;
+         this.titleElement.innerHTML = this.titleAbbrev + ' - ' + entry;
+      }
+   }
+   Object.assign(Navigation.prototype, WCom.Util.Bitch);
+   Object.assign(Navigation.prototype, WCom.Util.Markup);
+   Object.assign(Navigation.prototype, WCom.Util.String);
+   class Menus {
+      constructor(navigation, config) {
+         this.config        = config;
+         this.navigation    = navigation;
+         this.container     = navigation.container;
+         this.controlIcon   = navigation.controlIcon || 'settings';
+         this.icons         = navigation.icons;
+         this.linkDisplay   = navigation.linkDisplay;
+         this.location      = navigation.location;
+         this.token         = navigation.token;
+         this.contextPanels = {};
+         this.headerMenu;
+         this.globalMenu;
+      }
+      addSelected(item) {
+         item.classList.add('selected');
+         return true;
+      }
+      clickHandler(href, options) {
+         return function(event) {
+            event.preventDefault();
+            if (options.onUnload) options.onUnload();
+            else {
+               for (const cb of WCom.Util.Event.onunloadCallbacks()) cb();
+            }
+            if (options.renderLocation) options.renderLocation(href);
+            else this.navigation.renderLocation(href);
+         }.bind(this);
+      }
+      confirmHandler(name) {
+         return function(event) {
+            if (this.confirm) {
+               if (confirm(this.confirm.replace(/\*/, name))) return true;
+            }
+            else if (confirm()) return true;
+            event.preventDefault();
+            return false;
+         }.bind(this);
+      }
+      iconImage(icon) {
+         if (icon && icon.match(/:/)) return this.h.img({ src: icon });
+         else if (icon) {
+            const icons = this.icons;
+            if (!icons) return this.h.span({ className: 'text' }, '≡');
+            return this.h.icon({ className: 'icon', icons, name: icon });
+         }
+         return icon;
+      }
+      isCurrentHref(href) {
+         return history.state && history.state.href.split('?')[0]
+            == href.split('?')[0] ? true : false;
+      }
+      async loadMenuData(url) {
+         const state = { href: url + '' };
+         history.pushState(state, 'Unused', url); // API Darwin award
+         url.searchParams.set('navigation', true);
+         const { object } = await this.bitch.sucks(url);
+         if (!object || !object['menus']) return;
+         this.config = object['menus'];
+         this.token = object['verify-token'];
+         this.navigation.containerLayout = object['container-layout'];
+         this.navigation.titleEntry = object['title-entry'];
+      }
+      render() {
+         if (!this.config) return;
+         const content = [this.renderControl()];
+         if (!this.config['_global']) return;
+         const global = this.renderList(this.config['_global'], 'global');
+         if (this.location == 'header') content.unshift(global);
+         const cMenu = this.h.nav({ className: 'nav-menu' }, content);
+         this.headerMenu = this.display(this.container, 'headerMenu', cMenu);
+         if (this.location == 'header') return;
+         const container = document.getElementById(this.location);
+         const gMenu = this.h.nav({ className: 'nav-menu' }, global);
+         this.globalMenu = this.display(container, 'globalMenu', gMenu);
+      }
+      renderControl() {
+         if (!this.config['_control']) return;
+         const panelAttr = { className: 'nav-panel control-panel' };
+         const list = this.renderList(this.config['_control'], 'control');
+         this.contextPanels['control'] = this.h.div(panelAttr, list);
+         const controlAttr = { className: 'nav-control' };
+         const link = this.h.a(this.renderControlIcon());
+         return this.h.div(controlAttr, [link, this.contextPanels['control']]);
+      }
+      renderControlIcon() {
+         const icons = this.icons;
+         if (!icons)
+            return this.h.span({ className: 'nav-control-label text' }, '≡');
+         const name = this.controlIcon;
+         const icon = this.h.icon({
+            className: 'settings-icon', height: 24, icons, name, width: 24
+         });
+         return this.h.span({ className: 'nav-control-label' }, icon);
+      }
+      renderItem(item, menuName) {
+         const [text, href, icon] = item;
+         const iconImage = this.iconImage(icon);
+         const title = iconImage && this.linkDisplay == 'icon' ? text : '';
+         const itemAttr = { className: menuName, title };
+         if (typeof text != 'object') {
+            const label = this.renderLabel(icon, text);
+            if (href) {
+               const onclick = this.clickHandler(href, {});
+               const link = this.h.a({ href, onclick }, label);
+               link.setAttribute('clicklistener', true);
+               return this.h.li(itemAttr, link);
+            }
+            const labelAttr = { className: 'drop-menu' };
+            return this.h.li(itemAttr, this.h.span(labelAttr, label));
+         }
+         if (!text || text['method'] != 'post') return;
+         const verify = this.h.hidden({ name: '_verify', value: this.token });
+         const formAttr = { action: href, className: 'inline', method: 'post' };
+         const form = this.h.form(formAttr, verify);
+         form.addEventListener('submit', this.submitHandler(form, {}));
+         const onclick = this.confirmHandler(name);
+         const buttonAttr = { className: 'form-button', onclick };
+         const label = this.h.span(this.renderLabel(icon, text['name']));
+         form.append(this.h.button(buttonAttr, label));
+         return this.h.li(itemAttr, form);
+      }
+      renderLabel(icon, text) {
+         const iconImage = this.iconImage(icon);
+         return {
+            both: [iconImage, text],
+            icon: iconImage ? iconImage : text,
+            text: text
+         }[this.linkDisplay];
+      }
+      renderList(list, menuName) {
+         const [title, itemList] = list;
+         if (!itemList.length) return this.h.span({ className: 'empty-list' });
+         const items = [];
+         let context = false;
+         let isSelected = false;
+         for (const item of itemList) {
+            if (typeof item == 'string' && this.config[item]) {
+               let className = 'nav-panel';
+               if (menuName == 'context' || menuName == 'control')
+                  className = 'slide-out';
+               const rendered = this.renderList(this.config[item], 'context');
+               this.contextPanels[item] = this.h.div({ className }, rendered);
+               context = item;
+               continue;
+            }
+            const listItem = this.renderItem(item, menuName);
+            if (context) {
+               const panel = this.contextPanels[context];
+               if (panel.firstChild.classList.contains('selected'))
+                  isSelected = this.addSelected(listItem);
+               listItem.append(panel);
+               context = false;
+            }
+            if (this.isCurrentHref(item[1]))
+               isSelected = this.addSelected(listItem);
+            items.push(listItem);
+         }
+         const navList = this.h.ul({ className: 'nav-list' }, items);
+         if (menuName) navList.classList.add(menuName);
+         if (isSelected) navList.classList.add('selected');
+         return navList;
       }
       submitHandler(form, options = {}) {
          form.setAttribute('submitlistener', true);
          const action = form.action;
          return function(event) {
             event.preventDefault();
+            if (options.onUnload) options.onUnload();
+            else {
+               for (const cb of WCom.Util.Event.onunloadCallbacks()) cb();
+            }
             form.setAttribute('submitter', event.submitter.value);
-            this.process(action, form);
-            if (options.onSubmit) options.onSubmit();
+            this.navigation.process(action, form);
          }.bind(this);
       }
    }
-   Object.assign(Navigation.prototype, WCom.Util.Bitch);
-   Object.assign(Navigation.prototype, WCom.Util.Markup);
-   Object.assign(Navigation.prototype, WCom.Util.String);
+   Object.assign(Menus.prototype, WCom.Util.Bitch);
+   Object.assign(Menus.prototype, WCom.Util.Markup);
    class Messages {
       constructor(config) {
          this.bufferLimit = config['buffer-limit'] || 3;
